# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function, unicode_literals)

from django.core.exceptions import ImproperlyConfigured
from elasticsearch import Elasticsearch
from elasticsearch_dsl import Search
from rest_framework.response import Response


class ListElasticMixin(object):
    es_client = None

    es_model = None
    es_paginator = None
    es_filter_backends = ()
    es_excludes_fields = ()

    def get_es_client(self):
        """
        You may want to override this if you need to provide different
        Elasticsearch.client depending on the incoming request.
        """
        assert self.es_client is not None, (
            "'%s' should either include a `queryset` attribute, "
            "or override the `get_queryset()` method."
            % self.__class__.__name__
        )
        es_client = self.es_client
        if not isinstance(es_client, Elasticsearch):
            raise ValueError("Incorrect value es_client")
        return es_client

    def filter_search(self, search):
        for backend in list(self.es_filter_backends):
            search = backend().filter_search(self.request, search, self)
        return search

    def excludes_respond_fields(self, search):
        if self.es_excludes_fields:
            search = search.source(**{'excludes': self.es_excludes_fields})
        return search

    def get_es_search(self):
        if self.es_model is None:
            msg = (
                "Cannot use %s on a view which does not have the 'es_model'"
            )
            raise ImproperlyConfigured(msg % self.__class__.__name__)
        index = self.es_model()._get_index()
        es_client = self.get_es_client()
        s = Search(using=es_client, index=index, doc_type=self.es_model)
        return s

    @staticmethod
    def es_representation(iterable):
        """
        List of object instances
        """
        return [
            item.to_dict() for item in iterable
            ]

    def paginate_search(self, search):
        """
        Return a single page of results, or `None` if pagination is disabled.
        """
        if self.es_paginator is None:
            return None
        return self.es_paginator.paginate_search(search, self.request, view=self)

    def list(self, request, *args, **kwargs):
        search = self.filter_search(self.get_es_search())
        search = self.excludes_respond_fields(search)
        page = self.paginate_search(search)
        if page is not None:
            return self.es_paginator.get_paginated_response(
                self.es_representation(page)
            )
        return Response(self.es_representation(search.scan()))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function, unicode_literals)

from rest_framework import views

from .es_mixins import ListElasticMixin


class ListElasticAPIView(ListElasticMixin,
                         views.APIView):
    """
    Concrete view for listing a queryset.
    """
    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re

from setuptools import setup, find_packages


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    with open(os.path.join(package, '__init__.py'), 'rb') as init_py:
        src = init_py.read().decode('utf-8')
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", src).group(1)


setup(
    name='django-rest-elasticsearch',
    version=get_version('rest_framework_elasticsearch'),
    url='https://github.com/myarik/rest_framework_elasticsearch',
    license='BSD',
    description='Elasticsearch support for Django REST Framework',
    author='Yaroslav Muravskyi',
    author_email='y@myarik.com',

    packages=find_packages(
        where='.',
        exclude=('example_project*', )
    ),
    include_package_data=True,
    install_requires=[
        "Django>=1.8,<1.11",
        "djangorestframework>=3.2.0",
        "elasticsearch-dsl>=5.0.0,<6.0.0"
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function, unicode_literals)
from functools import reduce

from django.utils import six
from elasticsearch_dsl import Q, ValidationException

from rest_framework import filters
from rest_framework.settings import api_settings


class ESFieldFilter(object):
    def __init__(self, label, name=None):
        self.label = label
        self._name = name

    @property
    def name(self):
        if self._name is None:
            return self.label
        return self._name


class ElasticOrderingFilter(filters.OrderingFilter):
    def get_default_ordering(self, view):
        ordering = getattr(view, 'es_ordering', None)
        if isinstance(ordering, six.string_types):
            return (ordering,)
        return ordering

    def get_valid_fields(self, queryset, view, context={}):
        valid_fields = getattr(view, 'es_ordering_fields', self.ordering_fields)

        if valid_fields is None:
            # Default to allowing filtering on serializer fields
            return self.get_default_valid_fields(queryset, view, context)

        else:
            valid_fields = [
                (item, item) if isinstance(item, six.string_types) else item
                for item in valid_fields
                ]
        return valid_fields

    def filter_search(self, request, search, view):
        ordering = self.get_ordering(request, search, view)
        if ordering:
            search = search.sort(*ordering)
        return search


class ElasticFieldsFilter(object):
    @staticmethod
    def clean_field(field, data):
        # Hook for validate bolean
        if field.name == 'boolean':
            if data in (True, 'True', 'true', '1'):
                return True
            elif data in (False, 'False', 'false', '0'):
                return False
            else:
                return
        try:
            data = field.clean(data)
        except ValidationException:
            return
        # Hook for validate integer
        if field.name in ['short', 'integer', 'long']:
            try:
                data = int(data)
            except (ValueError, TypeError):
                return

        # Hook for validate float
        if field.name in ['float', 'double']:
            try:
                data = float(data)
            except (ValueError, TypeError):
                return
        return data

    def filter_search(self, request, search, view):
        es_filter_fields = getattr(view, 'es_filter_fields', None)
        es_model = getattr(view, 'es_model', None)
        if es_filter_fields:
            for item in es_filter_fields:
                try:
                    field = reduce(lambda d, key: d[key] if d else None,
                                   item.name.split('.'), es_model._doc_type.mapping)
                except KeyError:
                    # Incorrect field
                    continue
                args = request.query_params.get(item.label, '')
                data = [self.clean_field(field, value.strip()) for value in args.split(',')]
                # Remove empty string and None values
                data = [value for value in data
                        if value is not None and value is not '']
                if data:
                    search = search.filter('terms', **{item.name: data})
        return search


class ElasticSearchFilter(object):
    search_param = api_settings.SEARCH_PARAM
    search_should_match = '75%'

    def get_search_query(self, request):
        """
        Search terms are set by a ?search=... query parameter,
        and may be comma and/or whitespace delimited.
        """
        return request.query_params.get(self.search_param, '')

    def filter_search(self, request, search, view):
        search_fields = getattr(view, 'es_search_fields', None)
        search_query = self.get_search_query(request)

        if not search_fields or not search_query:
            return search
        q = Q("multi_match", query=search_query, fields=search_fields)
        search = search.query(q)
        search.query.minimum_should_match = self.search_should_match
        return search
